Package: ggtree
Type: Package
Title: an R package for visualization of tree and annotation data
Version: 4.0.4
Authors@R: c(
       person("Guangchuang", "Yu",     email = "guangchuangyu@gmail.com", role = c("aut", "cre", "cph"), 
            comment = c(ORCID = "0000-0002-6485-8781")),
       person("Tommy Tsan-Yuk", "Lam", email = "tylam.tommy@gmail.com",   role = c("aut", "ths")),
       person("Shuangbin", "Xu",       email = "xshuangbin@163.com",      role = "aut", 
            comment = c(ORCID="0000-0003-3513-5362")),
       person("Lin", " Li",            email = "williamlee2220@qq.com",   role = "ctb"),
       person("Bradley", "Jones",      email = "bjones@cfenet.ubc.ca",    role = "ctb"),
       person("Justin", "Silverman",   email = "jsilve24@gmail.com",      role = "ctb"),
       person("Watal M.", "Iwasaki",   email = "heavy.watal@gmail.com",   role = "ctb"),
       person("Yonghe", "Xia",         email = "xiayh17@gmail.com",       role = "ctb"),
       person("Ruizhu", "Huang",       email = "ruizhuRH@gmail.com",      role = "ctb")
       )
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Description: 'ggtree' extends the 'ggplot2' plotting system which implemented
    the grammar of graphics. 'ggtree' is designed for visualization and annotation
    of phylogenetic trees and other tree-like structures with their annotation data.
Depends: R (>= 4.2.0)
Imports: ape, aplot, dplyr, ggplot2 (>= 4.0.0), grid, magrittr,
        methods, purrr, rlang, ggfun (>= 0.1.7), yulab.utils (>=
        0.1.6), tidyr, tidytree (>= 0.4.5), treeio (>= 1.8.0), utils,
        scales, stats, cli, ggiraph (>= 0.9.1)
Suggests: emojifont, ggimage, ggplotify, shadowtext, grDevices, knitr,
        prettydoc, rmarkdown, testthat, tibble, glue, Biostrings
VignetteBuilder: knitr
ByteCompile: true
Encoding: UTF-8
License: Artistic-2.0
URL:
        https://www.amazon.com/Integration-Manipulation-Visualization-Phylogenetic-Computational-ebook/dp/B0B5NLZR1Z/
        (book),
        http://onlinelibrary.wiley.com/doi/10.1111/2041-210X.12628
        (paper)
BugReports: https://github.com/YuLab-SMU/ggtree/issues
Packaged: 2026-01-05 22:51:35 UTC; biocbuild
biocViews: Alignment, Annotation, Clustering, DataImport,
        MultipleSequenceAlignment, Phylogenetics, ReproducibleResearch,
        Software, Visualization
RoxygenNote: 7.3.3
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/ggtree
git_branch: RELEASE_3_22
git_last_commit: 8e25254
git_last_commit_date: 2026-01-04
Repository: Bioconductor 3.22
Date/Publication: 2026-01-05
NeedsCompilation: no
Author: Guangchuang Yu [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Tommy Tsan-Yuk Lam [aut, ths],
  Shuangbin Xu [aut] (ORCID: <https://orcid.org/0000-0003-3513-5362>),
  Lin Li [ctb],
  Bradley Jones [ctb],
  Justin Silverman [ctb],
  Watal M. Iwasaki [ctb],
  Yonghe Xia [ctb],
  Ruizhu Huang [ctb]
