# Generated by roxygen2: do not edit by hand

S3method(MRCA,ggtree)
S3method(collapse,ggtree)
S3method(fortify,agnes)
S3method(fortify,dendro)
S3method(fortify,dendrogram)
S3method(fortify,diana)
S3method(fortify,hclust)
S3method(fortify,igraph)
S3method(fortify,linkage)
S3method(fortify,multiPhylo)
S3method(fortify,obkData)
S3method(fortify,phylo)
S3method(fortify,phylo4)
S3method(fortify,phylo4d)
S3method(fortify,phylog)
S3method(fortify,phyloseq)
S3method(fortify,pvclust)
S3method(fortify,treedata)
S3method(fortify,treedataList)
S3method(fortify,twins)
S3method(ggplot_add,cladelab)
S3method(ggplot_add,cladelabel)
S3method(ggplot_add,facet_plot)
S3method(ggplot_add,facet_xlim)
S3method(ggplot_add,geom_range)
S3method(ggplot_add,ggexpand)
S3method(ggplot_add,hilight)
S3method(ggplot_add,layout_ggtree)
S3method(ggplot_add,range_xaxis)
S3method(ggplot_add,scale_ggtree)
S3method(ggplot_add,striplab)
S3method(ggplot_add,striplabel)
S3method(ggplot_add,taxalink)
S3method(ggplot_add,tiplab)
S3method(ggplot_add,tiplab_ylab)
S3method(ggplot_add,tree_inset)
S3method(ggplot_add,zoom_clade)
S3method(groupClade,ggtree)
S3method(groupOTU,ggtree)
S3method(nodeid,ggtree)
S3method(nodelab,ggtree)
S3method(offspring,ggtree)
S3method(print,ggtree)
export("%+>%")
export("%<%")
export("%<+%")
export("%>%")
export(Date2decimal)
export(GeomHilightEncircle)
export(GeomInteractiveCurvelink)
export(GeomInteractiveHilightEncircle)
export(GeomInteractiveHilightRect)
export(GeomInteractiveLabelGGtree)
export(GeomInteractivePointGGtree)
export(GeomInteractiveSegmentGGtree)
export(GeomInteractiveTextGGtree)
export(MRCA)
export(StatBalance)
export(add_colorbar)
export(aes)
export(arrow)
export(as.polytomy)
export(collapse)
export(decimal2Date)
export(expand)
export(facet_data)
export(facet_labeller)
export(facet_plot)
export(facet_widths)
export(flip)
export(fortify)
export(geom_aline)
export(geom_balance)
export(geom_cladelab)
export(geom_cladelabel)
export(geom_cladelabel2)
export(geom_facet)
export(geom_highlight)
export(geom_hilight)
export(geom_inset)
export(geom_label)
export(geom_label2)
export(geom_label2_interactive)
export(geom_motif)
export(geom_nodelab)
export(geom_nodelab2)
export(geom_nodepoint)
export(geom_point)
export(geom_point2)
export(geom_point2_interactive)
export(geom_range)
export(geom_rootedge)
export(geom_rootpoint)
export(geom_segment2)
export(geom_segment2_interactive)
export(geom_strip)
export(geom_striplab)
export(geom_taxalink)
export(geom_text)
export(geom_text2)
export(geom_text2_interactive)
export(geom_tiplab)
export(geom_tiplab2)
export(geom_tippoint)
export(geom_tree)
export(geom_tree2)
export(geom_treescale)
export(geom_zoom_clade)
export(get.path)
export(get_clade_position)
export(get_heatmap_column_position)
export(get_taxa_name)
export(ggdensitree)
export(ggexpand)
export(ggplot)
export(ggsave)
export(ggtree)
export(ggtree_set_interactive)
export(ggtree_unset_interactive)
export(gheatmap)
export(groupClade)
export(groupOTU)
export(guide_legend)
export(gzoom)
export(hexpand)
export(identify)
export(inset)
export(label_pad)
export(layout_circular)
export(layout_dendrogram)
export(layout_fan)
export(layout_inward_circular)
export(layout_rectangular)
export(margin)
export(msaplot)
export(multiplot)
export(nodebar)
export(nodeid)
export(nodelab)
export(nodepie)
export(open_tree)
export(plot_list)
export(range_format)
export(read.tree)
export(revts)
export(rotate)
export(rotate_tree)
export(rtree)
export(scaleClade)
export(scale_color)
export(scale_color_manual)
export(scale_color_subtree)
export(scale_colour_manual)
export(scale_colour_subtree)
export(scale_fill_manual)
export(scale_x_continuous)
export(scale_x_ggtree)
export(scale_x_range)
export(set_hilight_legend)
export(td_filter)
export(td_mutate)
export(td_unnest)
export(theme)
export(theme_dendrogram)
export(theme_inset)
export(theme_tree)
export(theme_tree2)
export(unit)
export(vexpand)
export(viewClade)
export(xlim)
export(xlim_expand)
export(xlim_tree)
export(zoomClade)
exportMethods(gzoom)
exportMethods(scale_color)
import(tidytree)
importFrom(ape,di2multi)
importFrom(ape,ladderize)
importFrom(ape,read.tree)
importFrom(ape,reorder.phylo)
importFrom(ape,rtree)
importFrom(aplot,plot_list)
importFrom(aplot,xrange)
importFrom(cli,cli_alert_warning)
importFrom(dplyr,collapse)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(ggfun,"%<+%")
importFrom(ggfun,get_aes_var)
importFrom(ggfun,identify)
importFrom(ggfun,td_filter)
importFrom(ggfun,td_mutate)
importFrom(ggfun,td_unnest)
importFrom(ggiraph,GeomInteractiveLabel)
importFrom(ggiraph,GeomInteractivePoint)
importFrom(ggiraph,GeomInteractivePolygon)
importFrom(ggiraph,GeomInteractiveRect)
importFrom(ggiraph,GeomInteractiveSegment)
importFrom(ggiraph,GeomInteractiveText)
importFrom(ggiraph,geom_text_interactive)
importFrom(ggiraph,girafe)
importFrom(ggiraph,opts_sizing)
importFrom(ggiraph,opts_toolbar)
importFrom(ggiraph,opts_tooltip)
importFrom(ggiraph,opts_zoom)
importFrom(ggplot2,Geom)
importFrom(ggplot2,GeomLabel)
importFrom(ggplot2,GeomPoint)
importFrom(ggplot2,GeomPolygon)
importFrom(ggplot2,GeomRect)
importFrom(ggplot2,GeomSegment)
importFrom(ggplot2,GeomText)
importFrom(ggplot2,Stat)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,annotation_custom)
importFrom(ggplot2,as_labeller)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,coord_polar)
importFrom(ggplot2,draw_key_path)
importFrom(ggplot2,draw_key_point)
importFrom(ggplot2,draw_key_polygon)
importFrom(ggplot2,draw_key_text)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,fortify)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_blank)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplotGrob)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,last_plot)
importFrom(ggplot2,layer)
importFrom(ggplot2,margin)
importFrom(ggplot2,position_nudge)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_discrete)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_date)
importFrom(ggplot2,scale_x_reverse)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,vars)
importFrom(ggplot2,waiver)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grid,arrow)
importFrom(grid,curveGrob)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,grid.layout)
importFrom(grid,grid.newpage)
importFrom(grid,grobTree)
importFrom(grid,pushViewport)
importFrom(grid,rectGrob)
importFrom(grid,unit)
importFrom(grid,viewport)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(magrittr,add)
importFrom(magrittr,equals)
importFrom(methods,is)
importFrom(methods,missingArg)
importFrom(methods,setGeneric)
importFrom(methods,setOldClass)
importFrom(rlang,.data)
importFrom(rlang,abort)
importFrom(rlang,as_name)
importFrom(rlang,check_installed)
importFrom(rlang,new_quosure)
importFrom(rlang,parse_expr)
importFrom(rlang,quo)
importFrom(rlang,quo_name)
importFrom(rlang,sym)
importFrom(scales,alpha)
importFrom(tidyr,gather)
importFrom(tidytree,MRCA)
importFrom(tidytree,ancestor)
importFrom(tidytree,as_tibble)
importFrom(tidytree,groupClade)
importFrom(tidytree,groupOTU)
importFrom(tidytree,nodeid)
importFrom(tidytree,nodelab)
importFrom(tidytree,offspring)
importFrom(tidytree,parent)
importFrom(treeio,Nnode)
importFrom(treeio,Ntip)
importFrom(treeio,as.phylo)
importFrom(treeio,as.treedata)
importFrom(treeio,read.fasta)
importFrom(utils,getFromNamespace)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(yulab.utils,get_cache_item)
importFrom(yulab.utils,get_fun_from_pkg)
importFrom(yulab.utils,update_cache_item)
importFrom(yulab.utils,yulab_msg)
