#' @importFrom ggfun %<+%
#' @export
#' @examples
#' nwk <- system.file("extdata", "sample.nwk", package="treeio")
#' tree <- read.tree(nwk)
#' p <- ggtree(tree)
#' dd <- data.frame(taxa=LETTERS[1:13],
#'                    place=c(rep("GZ", 5), rep("HK", 3), rep("CZ", 4), NA),
#'              value=round(abs(rnorm(13, mean=70, sd=10)), digits=1))
#' row.names(dd) <- NULL
#' p %<+% dd + geom_text(aes(color=place, label=label), hjust=-0.5)
ggfun::`%<+%`

#' @importFrom ggfun td_filter
#' @export 
ggfun::td_filter

#' @importFrom ggfun td_mutate
#' @export 
ggfun::td_mutate

#' @importFrom ggfun td_unnest 
#' @export 
ggfun::td_unnest 


#' @importFrom magrittr %>%
#' @export
magrittr::`%>%`

#' @importFrom ape rtree
#' @export
ape::rtree

#' @importFrom ape read.tree
#' @export
ape::read.tree

#' @importFrom tidytree groupOTU
#' @export
tidytree::groupOTU

#' @importFrom tidytree groupClade
#' @export
tidytree::groupClade

#' @importFrom tidytree nodeid
#' @export
tidytree::nodeid

#' @importFrom tidytree nodelab
#' @export
tidytree::nodelab

#' @importFrom tidytree MRCA
#' @export
tidytree::MRCA

#' @importFrom dplyr collapse
#' @export
dplyr::collapse

#' @importFrom ggplot2 fortify
#' @export
ggplot2::fortify

#' @importFrom ggplot2 ggplot
#' @export
ggplot2::ggplot

##' @importFrom ggplot2 xlim
##' @export
ggplot2::xlim

##' @importFrom ggplot2 theme
##' @export
ggplot2::theme

##' @importFrom ggplot2 ggsave
##' @export
ggplot2::ggsave

##' @importFrom ggplot2 aes
##' @export
ggplot2::aes

##' @importFrom ggplot2 geom_text
##' @export
ggplot2::geom_text

##' @importFrom ggplot2 geom_label
##' @export
ggplot2::geom_label

##' @importFrom ggplot2 geom_point
##' @export
ggplot2::geom_point

##' @importFrom ggplot2 guide_legend
##' @export
ggplot2::guide_legend

##' @importFrom ggplot2 scale_colour_manual
##' @export
ggplot2::scale_colour_manual

##' @importFrom ggplot2 scale_color_manual
##' @export
ggplot2::scale_color_manual

##' @importFrom ggplot2 scale_fill_manual
##' @export
ggplot2::scale_fill_manual

##' @importFrom ggplot2 scale_x_continuous
##' @export
ggplot2::scale_x_continuous

##' @importFrom ggplot2 margin
##' @export
ggplot2::margin

##' @importFrom grid arrow
##' @export
grid::arrow

##' @importFrom grid unit
##' @export
grid::unit


##' @importFrom aplot plot_list
##' @export
aplot::plot_list

##' @importFrom ggfun identify
##' @export
ggfun::identify
