% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_cladelabel.R
\name{geom_cladelabel}
\alias{geom_cladelabel}
\title{geom_cladelabel}
\usage{
geom_cladelabel(
  node,
  label,
  offset = 0,
  offset.text = 0,
  extend = 0,
  align = FALSE,
  barsize = 0.5,
  fontsize = 3.88,
  angle = 0,
  geom = "text",
  hjust = 0,
  color = NULL,
  fill = NA,
  family = "sans",
  parse = FALSE,
  horizontal = TRUE,
  ...
)
}
\arguments{
\item{node}{selected node}

\item{label}{clade label}

\item{offset}{offset of bar and text from the clade}

\item{offset.text}{offset of text from bar}

\item{extend}{extend bar height}

\item{align}{logical}

\item{barsize}{size of bar}

\item{fontsize}{size of text}

\item{angle}{angle of text}

\item{geom}{one of 'text' or 'label'}

\item{hjust}{justify text horizontally}

\item{color}{color for clade & label, of length 1 or 2}

\item{fill}{fill label background, only work with geom='label'}

\item{family}{sans by default, can be any supported font}

\item{parse}{logical, whether parse label}

\item{horizontal}{logical, whether set label to horizontal,
defaults to TRUE.}

\item{...}{additional parameter}
}
\value{
ggplot layers
}
\description{
annotate a clade with bar and text label
}
\seealso{
\link{geom_cladelabel2}
}
\author{
Guangchuang Yu
}
