% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{theme_dendrogram}
\alias{theme_dendrogram}
\title{theme_dendrogram}
\usage{
theme_dendrogram(bgcolor = "white", fgcolor = "black", ...)
}
\arguments{
\item{bgcolor}{set background color, defaults to "white"}

\item{fgcolor}{set color of axis}

\item{...}{additional parameter}
}
\description{
dendrogram theme
}
\author{
Guangchuang Yu
}
