% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_hilight.R
\docType{data}
\name{GeomInteractiveHilightEncircle}
\alias{GeomInteractiveHilightEncircle}
\title{ggproto classes for ggiraph}
\description{
ggproto classes for ggiraph
}
\keyword{datasets}
