% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_taxalink.R
\docType{data}
\name{GeomInteractiveCurvelink}
\alias{GeomInteractiveCurvelink}
\title{ggproto classes for ggiraph}
\description{
ggproto classes for ggiraph
}
\keyword{datasets}
