% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_interactive.R
\name{geom_segment2_interactive}
\alias{geom_segment2_interactive}
\title{Create interactive line segments of ggtree}
\usage{
geom_segment2_interactive(...)
}
\arguments{
\item{...}{see also the parameters of \code{geom_segment2()} of \code{ggtree}}
}
\description{
The geometry is based on \code{geom_segment2()}.
See the documentation for those functions for more details.
}
