% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/method-print.r
\name{ggtree_unset_interactive}
\alias{ggtree_unset_interactive}
\title{Unset ggtree interactive mode}
\usage{
ggtree_unset_interactive()
}
\description{
unset ggtree interactive mode
}
