% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_point.R
\docType{data}
\name{GeomInteractivePointGGtree}
\alias{GeomInteractivePointGGtree}
\title{ggproto classes for ggiraph}
\description{
ggproto classes for ggiraph
}
\keyword{datasets}
