% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree-utilities.R
\name{layoutDaylight}
\alias{layoutDaylight}
\title{Equal daylight layout method for unrooted trees.}
\usage{
layoutDaylight(model, branch.length, MAX_COUNT = 5)
}
\arguments{
\item{model}{tree object, e.g. phylo or treedata}

\item{branch.length}{set to 'none' for edge length of 1. Otherwise the phylogenetic tree edge length is used.}

\item{MAX_COUNT}{the maximum number of iterations to run (default 5)}
}
\value{
tree as data.frame with equal angle layout.
}
\description{
#' @title
}
\references{
The following aglorithm aims to implement the vague description of the "Equal-daylight Algorithm"
in "Inferring Phylogenies" pp 582-584 by Joseph Felsenstein.

\if{html}{\out{<div class="sourceCode">}}\preformatted{Leafs are subtrees with no children
Initialise tree using equal angle algorithm
tree_df = equal_angle(tree)

nodes = get list of nodes in tree_df breadth-first
nodes = remove tip nodes.

}\if{html}{\out{</div>}}
}
