% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_balance.R
\docType{data}
\name{StatBalance}
\alias{StatBalance}
\title{StatBalance}
\description{
StatBalance
}
\keyword{datasets}
