% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_segment.R
\docType{data}
\name{GeomInteractiveSegmentGGtree}
\alias{GeomInteractiveSegmentGGtree}
\title{ggproto classes for ggiraph}
\description{
ggproto classes for ggiraph
}
\keyword{datasets}
