% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_hilight.R
\docType{data}
\name{GeomHilightEncircle}
\alias{GeomHilightEncircle}
\title{ggproto classes of ggtree}
\description{
ggproto classes of ggtree
}
\keyword{datasets}
