% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_label.R
\docType{data}
\name{GeomInteractiveLabelGGtree}
\alias{GeomInteractiveLabelGGtree}
\title{ggproto classes for ggiraph}
\description{
ggproto classes for ggiraph
}
\keyword{datasets}
