% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_text.R
\docType{data}
\name{GeomInteractiveTextGGtree}
\alias{GeomInteractiveTextGGtree}
\title{ggproto classes for ggiraph}
\description{
ggproto classes for ggiraph
}
\keyword{datasets}
